/*
TASK: platforms
LANG: C++
*/

#include <cstdio>
#include <cstring>
#include <set>

struct bus
{
	int period;
	long long start;
	bus(){};
	bus(int _p, long long _s): period(_p), start(_s){};
};

struct node
{
	int x, y;
	long long time;
	node(){};
	node(int _x, int _y, long long _t): x(_x), y(_y), time(_t){};
};

inline bool operator< (const node &a, const node &b)
{
	return a.time != b.time ? a.time < b.time : (a.x != b.x ? a.x < b.x : a.y < b.y);
}

int N, M;
int st_x, st_y;
long long st_time;
int end_x, end_y;
long long h_first[55][55];
long long v_first[55][55];
bus horizontal[55], vertical[55];

inline int direction(int x, int y, long long time, bool horizontal)
{
	bus cur = horizontal ? ::horizontal[x] : ::vertical[y];
	int size = horizontal ? N : M;
	return ((((time - cur.start) / cur.period) / (size - 1)) & 1) ? -1 : 1;
}

inline int interval(int x, int y, bool horizontal)
{
	bus cur = horizontal ? ::horizontal[x] : ::vertical[y];
	int size = horizontal ? N : M;
	return 2 * (size - 1) * cur.period;
}

inline int half(int x, int y, bool horizontal)
{
	bus cur = horizontal ? ::horizontal[x] : ::vertical[y];
	int size = horizontal ? N : M;
	int pos = horizontal ? y : x;
	return 2 * (size - pos) * cur.period;
}

long long d[55][55];
int count[55][55];
std :: set<node> s;
long long dijkstra(const node &st, const node &end)
{
	memset(d, 127, sizeof(d));
	d[st.x][st.y] = st.time;
	s.insert(st);
	while(!s.empty())
	{
		node cur = *s.begin(); s.erase(s.begin());
		int x = cur.x, y = cur.y;
		long long time = cur.time;
		if(x == end.x && y == end.y)	return time;
		long long next, D;
		int add;
		//		HORIZONTAL	BEGINS
		if(time < h_first[x][y])	next = h_first[x][y];
		else
		{
			int interval_len = interval(x, y, 1);
			int interval_num = (time - h_first[x][y]) / interval_len;
			int interval_rem = (time - h_first[x][y]) % interval_len;
			if( interval_rem == 0)						next = interval_num * interval_len;
			else if( interval_rem <= half(x, y, 1) )	next = interval_num * interval_len + half(x, y, 1);
			else										next = (interval_num + 1) * interval_len;
			next += h_first[x][y];
		}
		add = direction(x, y, next, 1);
		next += horizontal[x].period;
		if( d[x][y + add] > next )
		{
			s.erase(node(x, y + add, d[x][y + add]));
			d[x][y + add] = next;
			s.insert(node(x, y + add, d[x][y + add]));
		}
		next -= horizontal[x].period;
		if( count[x][y] == 0 )
		{
			int dir = direction(x, y, next, 1);
			if(dir == 1)
				D = next + half(x, y, 1);
			else
				D = next + interval(x, y, 1) - half(x, y, 1);
			s.insert(node(x, y, D));
		}
		
		//		HORIZONTAL	ENDS
		
		//		VERTICAL	BEGINS
		if(time < v_first[x][y])	next = v_first[x][y];
		else
		{
			int interval_len = interval(x, y, 0);
			int interval_num = (time - v_first[x][y]) / interval_len;
			int interval_rem = (time - v_first[x][y]) % interval_len;
			if( interval_rem == 0 )						next = interval_num * interval_len;
			else if( interval_rem <= half(x, y, 0) )	next = interval_num * interval_len + half(x, y, 0);
			else										next = (interval_num + 1) * interval_len;
			next += v_first[x][y];
		}
		add = direction(x, y, next, 0);
		next += vertical[y].period;
		if( d[x + add][y] > next )
		{
			s.erase(node(x + add, y, d[x + add][y]));
			d[x + add][y] = next;
			s.insert(node(x + add, y, d[x + add][y]));
		}
		next -= vertical[y].period;
		if( count[x][y] == 0 )
		{
			int dir = direction(x, y, next, 0);
			if(dir == 1)
				D = next + half(x, y, 0);
			else
				D = next + interval(x, y, 0) - half(x, y, 0);
			s.insert(node(x, y, D));
		}
		//		VERTICAL	ENDS
		count[x][y] = 1;
	}
}

int main()
{
	scanf("%d %d\n", &M, &N);
	for(int i = 1; i <= M; i ++)
	{
		char c;
		scanf("%c %lld\n", &c, &horizontal[i].start);
		horizontal[i].period = (c == 'F') ? 1 : (c == 'O') ? 2 : 3;
	}
	for(int i = 1; i <= N; i ++)
	{
		char c;
		scanf("%c %lld\n", &c, &vertical[i].start);
		vertical[i].period = (c == 'F') ? 1 : (c == 'O') ? 2 : 3;
	}
	for(int i = 1; i <= M; i ++)
		for(int j = 1; j <= N; j ++)
			h_first[i][j] = horizontal[i].start + (j - 1) * horizontal[i].period,
			v_first[i][j] = vertical[j].start + (i - 1) * vertical[j].period;
	scanf("%d %d %lld %d %d", &st_x, &st_y, &st_time, &end_x, &end_y);
	printf("%lld\n", dijkstra(node(st_x, st_y, st_time), node(end_x, end_y, -1)));
    return 0;
}
