/*
ID: espr1t
LANG: C++
TASK: platforms
*/

#include <iostream>
#include <cstdio>
#include <cmath>
#include <vector>
#include <algorithm>
#include <cstdlib>
#include <string>
#include <queue>
#include <map>
#include <set>

#define MAX 64

using namespace std;
FILE *in; FILE *out;

struct State
{
	int time;
	int row, col;
	
	bool operator < (const State& r) const
	{
		if (time != r.time) return time > r.time;
		if (row != r.row) return row < r.row;
		return col < r.col;
	}
};

int n, m, d, e;
int rows[MAX], stimeRows[MAX];
int cols[MAX], stimeCols[MAX];

int vis[MAX][MAX];
priority_queue < State > q;


void getTimeAndDirection(int pos, int speed, int stime, int dim, int ctime, int& ntime, int& dir)
{
	pos--;
	int cpos, cycle = speed * (dim - 1) * 2;
	
	ntime = ctime;
	ctime -= stime;

	if (ctime < 0)
	{
		ntime = stime + speed * pos;
		dir = 1;
		if (pos == dim - 1) dir = -1;
	}
	else
	{
		ctime %= cycle;		
		dir = 1; cpos = 0;

		while (ctime > 0)
		{
			ctime -= speed; cpos += dir;
			if (cpos <= 0)       {dir = +1; cpos = 0;}
			if (cpos >= dim - 1) {dir = -1; cpos = dim - 1;}
		}
		
		ntime += -ctime;
		while (pos != cpos)
		{
			ntime += speed; cpos += dir;
			if (cpos <= 0)       {dir = +1; cpos = 0;}
			if (cpos >= dim - 1) {dir = -1; cpos = dim - 1;}
		}
	}
	return;
}

int dijkstra(int row, int col, int ctime)
{
	int i, c;
	int ntime, dir;
	State cur, next;
	
	memset(vis, 0, sizeof(vis));
	
	next.row = row, next.col = col;
	next.time = ctime;
	q.push(next);
	
	while(!q.empty())
	{
		cur = q.top(); q.pop();

		if (cur.row == d && cur.col == e) return cur.time;
		if (vis[cur.row][cur.col]) continue;
		else vis[cur.row][cur.col] = 1;
		
		// Taking the row platform
		getTimeAndDirection(cur.col, rows[cur.row], stimeRows[cur.row], n, cur.time, ntime, dir);
		
		next.row = cur.row;
		next.col = cur.col + dir;
		next.time = ntime + rows[cur.row];
		
		if (!vis[next.row][next.col]) q.push(next);
		
		// Taking the column platform
		getTimeAndDirection(cur.row, cols[cur.col], stimeCols[cur.col], m, cur.time, ntime, dir);

		next.row = cur.row + dir;
		next.col = cur.col;
		next.time = ntime + cols[cur.col];
		
		if (!vis[next.row][next.col]) q.push(next);
	}
	return -1;
}


int main(void)
{
	int i;
	int a, b, c;
	char type[32]; int stime;
	
	in = stdin; out = stdout;
//	in = fopen("platforms.in", "rt"); out = fopen("platforms.out", "wt");
	
	fscanf(in, "%d %d", &n, &m);
	
	for (i=1; i<=n; i++)
	{
		fscanf(in, "%s %d", type, &stime);
		if (type[0] == 'S') rows[i] = 3;
		if (type[0] == 'O') rows[i] = 2;
		if (type[0] == 'F') rows[i] = 1;
		stimeRows[i] = stime;
	}
	
	for (i=1; i<=m; i++)
	{
		fscanf(in, "%s %d", type, &stime);
		if (type[0] == 'S') cols[i] = 3;
		if (type[0] == 'O') cols[i] = 2;
		if (type[0] == 'F') cols[i] = 1;
		stimeCols[i] = stime;
	}
	fscanf(in, "%d %d %d %d %d", &a, &b, &c, &d, &e);
	
	int ans = dijkstra(a, b, c);
	fprintf(out, "%d\n", ans);	
	return 0;
}
