// Veselin Raychev SMG 11z

#include <stdio.h>
#include <math.h>

int n;

double tx1[20],ty1[20],tx2[20],ty2[20],tx3[20],ty3[20];
double px1,py1,px2,py2;

double ptx[62];
double pty[62];

double razst[62][62];

double bestway[62];

int crossprav(double x1,double y1,double x2,double y2,
              double x3,double y3,double x4,double y4)
{
  double s1,s2;

  s1 = x1*(y2-y3) + x2*(y3-y1) + x3*(y1-y2);
  s2 = x1*(y2-y4) + x2*(y4-y1) + x4*(y1-y2);

  if (fabs(s1)<1e-12) { return 0; } // Ne se presi4at
  if (fabs(s2)<1e-12) { return 0; }
  if ((s1*s2)>0) { return 0; }
  
//  if ((fabs(s1)>1e-9) && (fabs(s2)>1e-9) && (s1*s2<0)) { } else { return 0; }

  s1 = x3*(y4-y1) + x4*(y1-y3) + x1*(y3-y4);
  s2 = x3*(y4-y2) + x4*(y2-y3) + x2*(y3-y4);

  if (fabs(s1)<1e-12) { return 0; } // Ne se presi4at
  if (fabs(s2)<1e-12) { return 0; }
  if ((s1*s2)>0) { return 0; }

//  if ((fabs(s1)>1e-9) && (fabs(s2)>1e-9) && (s1*s2<0)) { return 1; }

  return 1;
}

int crosstriang(double x1,double y1,double x2,double y2)
{
  int i;
  for (i=0;i<n;i++)
  {
    if (crossprav(x1,y1,x2,y2, tx1[i],ty1[i], tx2[i],ty2[i])) { return 1; }
    if (crossprav(x1,y1,x2,y2, tx3[i],ty3[i], tx2[i],ty2[i])) { return 1; }
    if (crossprav(x1,y1,x2,y2, tx1[i],ty1[i], tx3[i],ty3[i])) { return 1; }    
  }
  return 0;
}

int buildrazst(void)
{
  int i,j,c;
  double raz,x1,y1,x2,y2;

  c = 3*n+2;
  for (i=0;i<c;i++)
  {
    x1=ptx[i];
    y1=pty[i];
  
    for (j=i+1;j<c;j++)
    {
      x2=ptx[j];
      y2=pty[j];
    
      if (crosstriang(x1,y1,x2,y2)==0) {
        x2=x1-x2;
        y2=y1-y2;
        raz = sqrt(x2*x2 + y2*y2);
      } else { raz=-2; }

      razst[i][j] = raz;
      razst[j][i] = raz;
    }
    razst[i][i] = -2;
  }
}

int finder(int going)
{
  int i,c;
  double r,bw;
  
  c = 3*n+2;

  bw = bestway[going];
  for (i=0;i<c;i++)
  {
    r = razst[going][i];
    if (r>-1)
    {
      if (bw+r<bestway[i])
      {
        bestway[i] = bw+r;
        finder(i);
      }
    }
  }
}

int findbestway(void)
{
  int i;
  for (i=1;i<62;i++)
  {
    bestway[i] = 1e+10;
  }
  bestway[0] = 0;
  
  finder(0);
}

int main(void)
{
  FILE* f;
  int i;
  f = fopen("short.inp","rt");
  fscanf(f,"%d",&n);
  for (i=0;i<n;i++)
  {
    fscanf(f,"%lf %lf %lf %lf %lf %lf",
          &tx1[i],&ty1[i],
          &tx2[i],&ty2[i],
          &tx3[i],&ty3[i]);
    ptx[i*3+2] = tx1[i];  pty[i*3+2] = ty1[i];
    ptx[i*3+3] = tx2[i];  pty[i*3+3] = ty2[i];
    ptx[i*3+4] = tx3[i];  pty[i*3+4] = ty3[i];
    
  }
  fscanf(f,"%lf %lf %lf %lf",&px1,&py1,&px2,&py2);

  ptx[0] = px1; pty[0] = py1;
  ptx[1] = px2; pty[1] = py2;
  
  fclose(f);

  buildrazst();
  findbestway();

  f = fopen("short.out","wt");  
  fprintf(f,"%.8lf\n",bestway[1]);
  fclose(f);

//  printf("%d\n",crossprav(0,0,1,1,0,0,1,1));
//  printf("%d\n",crossprav(1,0,0,1,0,0,1,1));

  
  return 0;
}
