Const
  GraphNdxMax = 200;
  OstMax = 10;
  AnsNdxMax = 1000;
  Infinity = MaxLongInt Div 2;
  NoRoute = 'No route';
  FinStr = '';
Type
  TAnsNdx = 0..AnsNdxMax;
  TGraphNdx = 0..GraphNdxMax;
  SGraphNdx = Set Of TGraphNdx;
  TOst = 0..OstMax;
  TNdx = LongInt;
  TData = LongInt;
  TNaslNode =
             Record
               Key:TNdx;
               Cena:TData;
             End;
  TNasl = Array[TGraphNdx]Of TNaslNode;
  TNode =
          Record
            Used:Boolean;
            NumNasl:TNdx;
            Nasl:TNasl;
          End;

  TGraph = Array[ TGraphNdx ] Of TNode;
  TAdd = Array[TGraphNdx, TOst ] Of SGraphNdx;
  TLight = Array[TGraphNdx ] Of TData;
  TParent = Array[TGraphNdx] Of TNdx;
  TLength = Array[TGraphNdx] Of TNdx;
  TDist = Array[TGraphNdx ] Of TData;
{  TUsed = Array[TGraphNdx]Of Boolean;}
  TAns = Array[TAnsNdx]Of TData;
Var
  Graph:TGraph;
  Add:TAdd;
  Light:TLight;
  Parent:TParent;
  Length:TLength;
  Dist:TDist;
  {Used:TUsed;}
  N, M, K:TNdx;
  Ans:TAns;
  BrAns:TNdx;

   Procedure Init;
     Begin
       BrAns:=0;
     End;

   Procedure AddEdge(Node1, Node2:TNdx; Cena:TData);
     Begin
       With Graph[Node1] Do
        Begin
          Inc(NumNasl);
          Nasl[NumNasl].Key:=Node2;
          Nasl[NumNasl].Cena:=Cena;
        End;
     End;

   Procedure InitDijkstra;
     Var
       I:TNdx;
     Begin
       For I:=1 To N Do
        Begin
          Parent[I]:=0;
          Length[I]:=0;
          Dist[I]:=Infinity;
          Graph[I].Used:=False;
        End;
     End;

   Procedure FindMinNode(Var MinNode:TNdx);
     Var
       I:TNdx;
       MinN:TNdx;
     Begin
       I:=1;
       While (I <= N) And (Graph[I].Used) Do
         Inc(I);
       MinN:=I;
       For I:=I + 1 To N Do
         If (Not Graph[I].Used) And (Dist[MinN] > Dist[I])  Then
            MinN:=I;
       MinNode:=MinN;
     End;

   Function FindNumNasl(Node1, Node2:TNdx) : TNdx;
     Var
       I:TNdx;
     Begin
       With Graph[Node1] Do
        Begin
          For I:=1 To NumNasl Do
            If Nasl[I].Key = Node2 Then
             Begin
               FindNumNasl:=I;
               Exit;
             End;
        End;
     End;

   Function Late( Node1, Node2:TNdx; Len:TNdx) : TData;
     Var
       Late_:TData;
       I, J:TNdx;
     Begin
       I:=FindNumNasl(Node1, Node2);
       Late_:=Graph[Node1].Nasl[I].Cena;
       If Not (Node1 In Add[Node2, Len Mod K]) Then
         Inc(Late_, Light[Node2]);
       Late:=Late_;
     End;

   Procedure ReDist(Node:TNdx);
     Var
       I:TNdx;
       Late_:TData;
     Begin
       With Graph[Node] Do
        Begin
          For I:=1 To NumNasl Do
            If Not Graph[ Nasl[I].Key ].Used Then
             Begin
               Late_:=Late(Node, Nasl[I].Key, Length[Node] + 1);
               If (Dist[Nasl[I].Key] > ( Dist[Node] + Late_) ) Then
                Begin
                  Dist[Nasl[I].Key]:=Dist[Node] + Late_;
                  Parent[Nasl[I].Key]:=Node;
                  Length[Nasl[I].Key]:=Length[Node] + 1;
                End;
             End;
        End;
     End;

   Procedure Dijkstra(Node1, Node2:TNdx);
     Var
       Done:Boolean;
       I, MinNode:TNdx;
     Begin
       { Init }
       Graph[Node1].Used:=True;
       Dist[Node1]:=0;
       With Graph[Node1] Do
          For I:=1 To NumNasl Do
            Dist[ Nasl[I].Key ]:=Nasl[I].Cena;


       Done:=False;
       While Not Done Do
        Begin
          FindMinNode(MinNode);
          Graph[MinNode].Used:=True;
          If MinNode = Node2 Then
            Done:=True
          Else ReDist(MinNode);
        End;
     End;

   Function Razt(Node2:TNdx):TData;
     Begin
       If Dist[Node2] = Infinity Then
         Razt:=-1
       Else
         Razt:=Dist[Node2];
     End;

   Procedure ReadNSolve;
     Var
       Num, Cena:TData;
       I, J, Node1, Node2:TNdx;
       Fin:Text;
     Begin
       Assign(Fin, FinStr);
       ReSet(Fin);
       ReadLn(Fin, N, M, K);
       For I:=1 To N Do
        Begin
          ReadLn(Fin, Light[I]);
          For J:=0 To K - 1 Do
           Begin
             Read(Fin, Num);
             While Num <> -1 Do
              Begin
                Add[I, J]:= Add[I, J] + [Num];
                Read(Fin, Num);
              End;
             ReadLn(Fin);
           End;
        End;
       For I:=1 To M Do
        Begin
          ReadLn(Fin, Node1, Node2, Cena);
          AddEdge(Node1, Node2, Cena);
          AddEdge(Node2, Node1, Cena);
        End;
       While (Not Eof(Fin)) Or (Not Eoln(Fin)) Do
        Begin
          ReadLn(Fin, Node1, Node2);
          InitDijkstra;
          Dijkstra(Node1, Node2);
          Inc(BrAns);
          Ans[BrAns]:=Razt(Node2);
        End;

       Close(Fin);
     End;

   Procedure Print;
     Var
       I:TNdx;
     Begin
       For I:=1 To BrAns Do
        If Ans[I] = -1 Then
          WriteLn(NoRoute)
        Else
          WriteLn(Ans[I]);
     End;

Begin
  Init;
  ReadNSolve;
  Print;
End.
