/*
TASK: Streets(solver)
LANG: C++
AUTHOR: Alexander Georgiev
COMPETITION: Winter Mathematical Festivals
*/

#include <algorithm>
#include <cstdio>
#include <cstring>
#include <vector>
#include <queue>

#define MAX 512
#define INF 1000000000
#define MASK 2097152

using namespace std;
FILE *in; FILE *out;

struct State
{
	int node, last, price;
	bool operator < (const State& r) const
	{
		if (price != r.price) return price > r.price;
		return node != r.node ? node < r.node : last < r.last;
	}
};

int n, m;
char part[MAX];
vector <int> v[MAX], r[MAX];
int vis[MAX], last[MAX];
int ma3x[MAX][MAX], prices[MAX][MAX], num[MAX][MAX], next[MAX][MAX];

int a[MAX][2], len;
int dyn[MASK], save[MASK];
vector <int> path;

void init()
{
	n = 0; m = 0;
	memset(part, 0, sizeof(part));
	memset(vis, 0, sizeof(vis));
	memset(last, 0, sizeof(last));
	memset(ma3x, 0, sizeof(ma3x));
	memset(prices, 0, sizeof(prices));
	memset(num, 0, sizeof(num));
	memset(next, 0, sizeof(next));
	for (int i = 0; i < MAX; i++) v[i].clear();
	for (int i = 0; i < MAX; i++) r[i].clear();
	
	memset(a, 0, sizeof(a)); len = 0;
	memset(dyn, -1, sizeof(dyn));
	memset(save, -1, sizeof(save));
	path.clear();
}

void findEulerian(int node)
{
	for (int i = 0; i < (int)v[node].size(); i++)
	{
		if (num[node][v[node][i]])
		{
			num[node][v[node][i]]--;
			findEulerian(v[node][i]);
		}
	}
	path.push_back(node);
}

void addEdge(int from, int to, int price)
{
	v[from].push_back(to);
	r[to].push_back(from);
	ma3x[from][to] = price;
	num[from][to]++;
}

void addPath(int from, int to)
{
	while (from != to)
	{
		v[from].push_back(next[from][to]);
		r[next[from][to]].push_back(from);
		num[from][next[from][to]]++;
		from = next[from][to];
	}
}

int recurse(int cur, int mask)
{
	if (cur >= len) return 0;
	if (dyn[mask] != -1) return dyn[mask];
	
	int ans = INF, nxt = -1;
	for (int i = 0; i < len; i++) if ((mask & (1 << i)) == 0)
	{
		int curPrice = recurse(cur + 1, mask | (1 << i)) + prices[a[cur][0]][a[i][1]];
		if (ans > curPrice) ans = curPrice, nxt = i;
	}
	dyn[mask] = ans; save[mask] = nxt;
	return ans;
}

void doDynamic()
{
	int idx1 = 0, idx2 = 0;
	for (int node = 0; node < n; node++)
	{
		for (int i = (int)v[node].size(); i < (int)r[node].size(); i++)
			a[idx1++][0] = node;
		for (int i = (int)r[node].size(); i < (int)v[node].size(); i++)
			a[idx2++][1] = node;
	}
	
	len = idx1;
	int price = recurse(0, 0);
	
	int cur = 0, mask = 0;
	while (cur < len)
	{
		int node1 = a[cur][0];
		int node2 = a[save[mask]][1];
		addPath(node1, node2);
		mask |= (1 << save[mask]); cur++;
	}
}

void buildTree(int root)
{
	State cur, add;
	memset(vis, 0, sizeof(vis));
	priority_queue < State > q;
	add.node = root; add.last = -1; add.price = 0; q.push(add);
	
	while(!q.empty())
	{
		cur = q.top(); q.pop();
		
		if (vis[cur.node]) continue; vis[cur.node] = 1;
		next[root][cur.node] = cur.last;
		prices[root][cur.node] = cur.price;
		
		for (int i = 0; i < (int)v[cur.node].size(); i++) if (!vis[v[cur.node][i]])
		{
			add.node = v[cur.node][i];
			add.last = cur.node == root ? add.node : cur.last;
			add.price = cur.price + ma3x[cur.node][add.node];
			q.push(add);
		}
	}	
}

void findPrices()
{
	memset(prices, 127, sizeof(prices));
	memset(next, -1, sizeof(next));
	for (int node = 0; node < n; node++)
		buildTree(node);
}

int fill(int node)
{
	vis[node] = 1;
	int ans = (int)v[node].size();
	for (int i = 0; i < (int)v[node].size(); i++)
		if (!vis[v[node][i]]) ans += fill(v[node][i]);
	return ans;
}

int isImpossible()
{
	memset(vis, 0, sizeof(vis));
	int numEdges = fill(0);
	if (numEdges != m) return 1;
	memcpy(last, vis, sizeof(vis));
	for (int node = 1; node < n; node++) if (last[node])
	{
		memset(vis, 0, sizeof(vis));
		fill(node);
		if (!vis[0]) return 1;
	}
	return 0;
}

void solve(char* inpFile, char* outFile)
{
	in = fopen(inpFile, "rt"); out = fopen(outFile, "wt");
	
	init();
	fscanf(in, "%d %d", &n, &m);
	for (int i = 0; i < m; i++)
	{
		int from, to, price;
		fscanf(in, "%d %d %d", &from, &to, &price);
		addEdge(from, to, price);
	}
	fscanf(in, "%s", part);
	
	if (isImpossible()) fprintf(out, "-1\n");
	else
	{
		findPrices();
		doDynamic();
		findEulerian(0);
		
		int sum = 0;
		for (int node = 0; node < n; node++)
			for (int i = 0; i < (int)v[node].size(); i++)
				sum += ma3x[node][v[node][i]];
		
		fprintf(out, "%d\n", sum);
		if (!strcmp(part, "full") && sum > 0)
		{
			reverse(path.begin(), path.end());
			fprintf(out, "%d\n", (int)path.size());
			for (int i = 0; i < (int)path.size(); i++)
				fprintf(out, "%d%c", path[i], i == (int)path.size() - 1 ? '\n' : ' ');
		}
	}
	fclose(in); fclose(out);
	return;
}

int main(void)
{
	int numTests = 20;
	
	for (int test = 1; test <= numTests; test++)
	{
		fprintf(stderr, "Solving test Nr.%02d...\n", test);
		char inpFile[32], outFile[32];
		sprintf(inpFile, "streets.%02d.in", test);
		sprintf(outFile, "streets.%02d.sol", test);
		solve(inpFile, outFile);
	}
	return 0;
}
