import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;

/**
 * Solution of the problem: Alea iacta est, five-dice version
 * Martin Kacer, July 2008
 *
 * The solution itself has about 200 lines of source code.
 * This file is a little bit longer because it includes comments,
 * it also prints some debug info and a complete path to the best
 * result. See DEBUG constant for this.
 */
public class alea
{
	static final int NUMDICE = 5; // five-dice version

	/**
	 * One "throw" holds the result of one round.
	 */
	static class OneThrow {
		int cnt[] = new int[6]; // number of ones, twos, etc.
		int consume; // how many random numbers have been used
		int scores[]; // pre-computed scores for all existing combinations
		@Override public boolean equals(Object obj) {
			if (!(obj instanceof OneThrow)) return false;
			return this.hashCode() == obj.hashCode();
		}
		@Override public int hashCode() {
			int hc = consume;
			for (int x : cnt) hc = hc * 7 + x;
			return hc;
		}
		@Override public String toString() {
			String s = "";
			for (int i = 0; i < 6; ++i)
				for (int c = 0; c < cnt[i]; ++c) {
					s += (i+1);
				}
			return s + ":" + consume;
		}
	}

	static abstract class Combination {
		abstract int score(OneThrow th);
		int sum(OneThrow th) {
			int sum = 0;
			for (int i = 0; i < th.cnt.length; ++i) {
				sum += th.cnt[i] * (i+1);
			}
			return sum;
		}
	}

	static class CombNumbers extends Combination {
		int num;
		CombNumbers(int n) {this.num = n;}
		int score(OneThrow th) {return num * th.cnt[num-1];}
	}

	static class CombSequence extends Combination {
		int score(OneThrow th) {
			for (int x : th.cnt) if (x > 1) return 0;
			return (th.cnt[0] == 0 || th.cnt[5] == 0) ? 30 : 0;
		}
	}

	static class CombFullHouse extends Combination {
		int score(OneThrow th) {
			for (int x : th.cnt) {
				if (x != 0 && x != 2 && x != 3) return 0;
			}
			return sum(th);
		}
	}

	static class CombFour extends Combination {
		int score(OneThrow th) {
			boolean ok = false;
			for (int x : th.cnt) {
				if (x == 4) ok = true;
			}
			return ok ? sum(th) : 0;
		}
	}

	static class CombAllSame extends Combination {
		int score(OneThrow th) {
			for (int x : th.cnt) {
				if (x != 0 && x != NUMDICE) return 0;
			}
			return 50;
		}
	}

	static class CombChance extends Combination {
		int score(OneThrow th) {
			return sum(th);
		}
	}


	static final boolean DEBUG = false;
	static final Combination[] COMB = new Combination[]{
		new CombNumbers(1), new CombNumbers(2), new CombNumbers(3),
		new CombNumbers(4), new CombNumbers(5), new CombNumbers(6),
		new CombSequence(), new CombFullHouse(), new CombFour(),
		new CombAllSame(), new CombChance()
	};
	static final int MAX_THROW = COMB.length * NUMDICE * 3;
	static final int BIT_COMB = 1 << COMB.length;

	int randoms[] = new int[MAX_THROW];

	/** All possible selections of dice in one round. */
	int numsel = 0;
	int selections[][] = new int[729][NUMDICE]; // 729 is for six dice

	/** Dynamic-programming array. All variants of the set of
	 * combinations used and number of randoms consumed. */
	int sol[][] = new int[BIT_COMB][MAX_THROW+1];

	/** Only needed to reconstruct the resulting path! */
	OneThrow usedthrow[][] = new OneThrow[BIT_COMB][MAX_THROW+1];
	int usedcomb[][] = new int[BIT_COMB][MAX_THROW+1];

	/**
	 * Recursive method to generate all achievable selections of the dice.
	 * @param startidx how many randoms have been used before
	 * @param idxs array to store indexes of selected randoms
	 * @param rest how many dice to be generated yet
	 * @param thrleft how many throws are left
	 */
	void findRest(int startidx, int[] idxs, int rest, int thrleft) {
		int valid = NUMDICE - rest;
		if (thrleft == 1) {
			// last throw => we must take everything
			for (int i = 0; i < NUMDICE-rest; ++i) selections[numsel][i] = idxs[i];
			for (int i = 0; i < rest; ++i) selections[numsel][valid + i] = startidx + i;
			++numsel;
			return;
		}
		int comb = 1 << rest;
		for (int mask = 0; mask < comb; ++mask) {
			int v = valid;
			for (int i = 0; i < rest; ++i) {
				if (((1<<i) & mask) != 0) {
					idxs[v++] = startidx + i;
				}
			}
			findRest(startidx + rest, idxs, NUMDICE-v, thrleft-1);
		}
	}
	
	/**
	 * Fill the "selection" array. Called at the startup only.
	 */
	void findSelections() {
		int idxs[] = new int[NUMDICE];
		findRest(0, idxs, NUMDICE, 3);
	}

	/**
	 * Generate and return the set of all possible throws.
	 * @param startidx index of the first random number to be used.
	 */
	Set<OneThrow> genThrows(int startidx) {
		Set<OneThrow> res = new HashSet<OneThrow>();
		for (int i = 0; i < numsel; ++i) {
			OneThrow th = new OneThrow();
			for (int j = 0; j < NUMDICE; ++j) {
				++ th.cnt[randoms[startidx + selections[i][j]]];
				th.consume = selections[i][NUMDICE-1] + 1;
			}
			if (res.add(th)) {
				th.scores = new int[COMB.length];
				for (int c = 0; c < COMB.length; ++c)
					th.scores[c] = COMB[c].score(th);
			}
		}
		return res;
	}

	void run() throws Exception {
		findSelections();
		for (;;) {
			int a = nextInt();
			int c = nextInt();
			int x = nextInt();
			if (a == 0) break;
			oneprob(a, c, x);
		}
	}

	/**
	 * Solve one task.
	 */
	void oneprob(int aa, int cc, int x0) {
		long x = x0;
		for (int i = 0; i < randoms.length; ++i) {
			x = (x * aa + cc) & 0xFFFFFFFFL;
			randoms[i] = (int) ((x>>>16) % 6);
		}
		for (int i = 0; i < BIT_COMB; ++i)
			for (int j = 0; j <= MAX_THROW; ++j)
				sol[i][j] = -1;
		sol[0][0] = 0;
		for (int j = 0; j <= MAX_THROW - 18; ++j) {
			if (DEBUG) System.out.println("--> " + j);
			Set<OneThrow> thrs = genThrows(j);
			OneThrow[] athrs = thrs.toArray(new OneThrow[thrs.size()]);
			for (int i = 0; i < BIT_COMB; ++i)
				if (sol[i][j] >= 0) {
					expand(i, j, athrs);
				}
		}
		int best = 0, bestj = 0;
		for (int j = 0; j <= MAX_THROW; ++j) {
			if (sol[BIT_COMB-1][j] > best) {
				best = sol[BIT_COMB-1][j];
				bestj = j;
			}
		}
		System.out.println(best);
		
		if (DEBUG) {
			for (int i = 0; i < MAX_THROW; ++i) {
				System.out.print(randoms[i]+1 + " ");
			}
			System.out.println();
			showSolution(BIT_COMB-1, bestj);
		}
	}

	/**
	 * Expand all possibilities from the given state.
	 * @param thrs pre-computed array of all possible throws.
	 */
	void expand(int combmask, int startidx, OneThrow[] thrs) {
		int myscore = sol[combmask][startidx];
		for (OneThrow th : thrs) {
			int y = startidx + th.consume;
			for (int i = 0, ib = 1; i < COMB.length; ++i, ib<<=1)
				if ((combmask & ib) == 0) {
					int sc = myscore + th.scores[i];
					int x = combmask | ib;
					if (sol[x][y] < sc) {
						sol[x][y] = sc;
						usedthrow[x][y] = th;
						usedcomb[x][y] = i;
					}
				}
		}
	}

	/** Reconstruct the path and print it. */
	void showSolution(int combmask, int endidx) {
		if (endidx == 0) return;
		int c = usedcomb[combmask][endidx];
		OneThrow th = usedthrow[combmask][endidx];
		showSolution(combmask & ~(1<<c), endidx - th.consume);
		System.out.print("idx=" + (endidx - th.consume) + ": " + th);
		System.out.print(" comb=" + c + " (" + th.scores[c] + ") ");
		for (int i = 0; i < th.consume; ++i) {
			System.out.print(" " + (randoms[endidx - th.consume + i]+1));
		}
		System.out.println();
	}

	StringTokenizer st = new StringTokenizer("");
	BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
	public static void main(String[] args) throws Exception {
		new alea().run();
	}
	String nextToken() throws Exception {
		while (!st.hasMoreTokens()) st = new StringTokenizer(input.readLine());
		return st.nextToken();
	}
	int nextInt() throws Exception {
		return Integer.parseInt(nextToken());
	}
}
