#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
 * CTU Open Contest 2008
 * Solution: BANKING
 * Jan Stoklasa, Oct 2008 
 */


#define EPS 0.00005


class Accounts
{
public:

  Accounts(int MaxSize);

  ~Accounts();

  static const int MAXLEN=1000;
  typedef char str[MAXLEN];

  enum CreateResult {CR_OK, AlreadyExists};
  CreateResult create(str name,float balance);

  enum DepositResult {DR_OK, DR_NoSuchAccount};
  DepositResult deposit(str name,float money);

  enum WithdrawResult {WR_OK,  WR_NoSuchAccount, WR_InsufficientFunds};
  WithdrawResult withdraw(str from,float money);

  enum TransferResult {TR_OK, Interbank, InsufficientFunds,
TR_NoSuchAccount, TR_SameAccount };
  TransferResult transfer(str from,str to,float money);

private:

  struct account
  {
    char name[50];
    float balance;
  };

  account* accounts;
  int accounts_ptr;

  int find(str name);

};


char* read(Accounts::str line)
{
  if(fgets(line,Accounts::MAXLEN-1,stdin)==NULL)
    return NULL;
  int last=strlen(line)-1;
  if(line[last]=='\n')
    {
      line[last]='\0';
    }
  return line;
};

const int MaxAccounts=100;
const int MaxOperations=1000;
int main()
{
  Accounts::str line;
  int A;
  while(read(line)&&
	(A=atoi(line))>0)
    {
      Accounts a(MaxAccounts+MaxOperations);
      for(int i=0;i<A;i++)
	{
	  Accounts::str account;
	  Accounts::str name;
	  float balance=11;
	  read(account);
	  sscanf(account,"%s %f",name,&balance);
	  a.create(name,balance);
	}
      while(read(line)&&
	    strcmp(line,"end")!=0)
	{
	  Accounts::str command;
	  Accounts::str name;
	  float money;	
	  sscanf(line,"%s ",command);
	  if(strcmp(command,"create")==0)
	    {	
	      sscanf(line,"%s %s",command,name);
	      Accounts::CreateResult cr=a.create(name,0);
	      if(cr==Accounts::CR_OK)
		{
		  printf("%s: ok\n",command);
		}
	      else if(cr==Accounts::AlreadyExists)
		{
		  printf("%s: already exists\n",command);
		}
	    }
	  else if(strcmp(command,"deposit")==0)
	    {	
	      sscanf(line,"%s %s %f",command,name,&money);
	      Accounts::DepositResult dr=a.deposit(name,money);
	      if(dr==Accounts::DR_OK)
		{
		  printf("%s %.2f: ok\n",command,money);
		}
	      else if(dr==Accounts::DR_NoSuchAccount)
		{
		  printf("%s %.2f: no such account\n",command,money);
		}
	    }
	  else if(strcmp(command,"withdraw")==0)
	    {
	      sscanf(line,"%s %s %f",command,name,&money);
	      Accounts::WithdrawResult wr=a.withdraw(name,money);
	      if(wr==Accounts::WR_OK)
		{
		  printf("%s %.2f: ok\n",command,money);
		}
	      else if(wr==Accounts::WR_NoSuchAccount)
		{
		  printf("%s %.2f: no such account\n",command,money);
		}
	      else if(wr==Accounts::WR_InsufficientFunds)
		{
		  printf("%s %.2f: insufficient funds\n",command,money);
		}	
	    }
	  else if(strcmp(command,"transfer")==0)
	    {
	      Accounts::str to;
	      sscanf(line,"%s %s %s %f",command,name,to,&money);
	      Accounts::TransferResult tr=a.transfer(name,to,money);
	      if(tr==Accounts::TR_OK)
		{
		  printf("%s %.2f: ok\n",command,money);
		}
	      else if(tr==Accounts::Interbank)
		{
		  printf("%s %.2f: interbank\n",command,money);
		}
	      else if(tr==Accounts::InsufficientFunds)
		{
		  printf("%s %.2f: insufficient funds\n",command,money);
		}
	      else if(tr==Accounts::TR_NoSuchAccount)
		{
		  printf("%s %.2f: no such account\n",command,money);
		}
	      else if(tr==Accounts::TR_SameAccount)
	      {
	         printf("%s %.2f: same account\n",command,money);
	      }
	    }
	}
     /*
     for(int i=0;i<accounts_ptr;i++)
       {
	 printf("%s %f\n",accounts[i].name,accounts[i].balance);
       }
     */
      printf("end\n\n");
      read(line); //skip empty line
    }
  printf("%s\n","goodbye");
  return EXIT_SUCCESS;
}

int Accounts::find(str name)
{
  for(int i=0;i<accounts_ptr;i++)
    {
      if(strcmp(accounts[i].name,name)==0)
	{
	  return i;
	}
    }
  return -1;
}

Accounts::Accounts(int MaxSize)
{
  accounts_ptr=0;
  accounts=new account[MaxSize];
}

Accounts::~Accounts()
{
  delete accounts;
}

Accounts::CreateResult Accounts::create(str name,float balance)
{
  int i=find(name);
  if(i<0)
    {
      strcpy(accounts[accounts_ptr].name,name);
      accounts[accounts_ptr].balance=balance;
      accounts_ptr++;
      return CR_OK;
    }
  else
    {
      return AlreadyExists;
    }
}

Accounts::DepositResult Accounts::deposit(str name,float money)
{
  int i=find(name);
  if(i>=0)
    {
      accounts[i].balance+=money;
      return DR_OK;
    }
  else
    {
      return DR_NoSuchAccount;
    }
}

Accounts::WithdrawResult Accounts::withdraw(str from,float money)
{
  int f=find(from);
  if(f<0)
    {
      return WR_NoSuchAccount;
    }
  if(accounts[f].balance+EPS<money)
    {
      return WR_InsufficientFunds;
    }
  accounts[f].balance-=money;
  return WR_OK;
}

Accounts::TransferResult Accounts::transfer(str from,str to,float money)
{
  int f=find(from);
  int t=find(to);
  if(f<0 || t<0)
    {
      return TR_NoSuchAccount;
    }

  if(f==t)
    {
      return TR_SameAccount;	
    }


  if(accounts[f].balance+EPS<money)
    {
      return InsufficientFunds;
    }

  accounts[f].balance-=money;
  accounts[t].balance+=money;

  if(from[strlen(from)-1]!=to[strlen(to)-1])
    {
      return Interbank;
    }
  else
    {
      return TR_OK;
    }
}
