import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

/**
 * CTU Open Contest 2008
 * Solution for the problem: BANKING.
 * October, 2008
 *
 * @author Martin Kacer
 */
public class banking {
	Map<String,Integer> acc;

	static abstract class Command {abstract String go();}

	Map<String,Command> cmd = new HashMap<String,Command>();
	{
		cmd.put("create", new Command() { String go() {
			String a = nextToken();
			System.out.print("create: ");
			if (acc.containsKey(a)) return "already exists";
			acc.put(a, Integer.valueOf(0));
			return "ok";
		}});
		cmd.put("deposit", new Command() { String go() {
			String a = nextToken();
			int v = fromString(nextToken());
			System.out.print("deposit " + makeString(v) + ": ");
			Integer old = acc.get(a);
			if (old == null) return "no such account";
			acc.put(a, Integer.valueOf(old.intValue() + v));
			return "ok";
		}});
		cmd.put("withdraw", new Command() { String go() {
			String a = nextToken();
			int v = fromString(nextToken());
			System.out.print("withdraw " + makeString(v) + ": ");
			Integer old = acc.get(a);
			if (old == null) return "no such account";
			if (old.intValue() < v) return "insufficient funds";
			acc.put(a, Integer.valueOf(old.intValue() - v));
			return "ok";
		}});
		cmd.put("transfer", new Command() { String go() {
			String a1 = nextToken();
			String a2 = nextToken();
			int v = fromString(nextToken());
			System.out.print("transfer " + makeString(v) + ": ");
			Integer old1 = acc.get(a1);
			Integer old2 = acc.get(a2);
			if (old1 == null || old2 == null) return "no such account";
			if (a1.equals(a2)) return "same account";
			if (old1.intValue() < v) return "insufficient funds";
			acc.put(a1, Integer.valueOf(old1.intValue() - v));
			acc.put(a2, Integer.valueOf(old2.intValue() + v));
			return a1.charAt(5) == a2.charAt(5) ? "ok" : "interbank";
		}});
	}

	boolean run() throws Exception {
		int n = nextInt();
		if (n == 0) return false;
		acc = new HashMap<String,Integer>();
		while (n-- > 0) {
			String a = nextToken();
			acc.put(a, Integer.valueOf(fromString(nextToken())));
		}
		for (;;) {
			Command c = cmd.get(nextToken());
			if (c == null) {
				System.out.println("end");
				System.out.println();
				return true;
			}
			System.out.println(c.go());
		}
	}

	int fromString(String s) {
		return Integer.parseInt(s.replaceAll("[.]", ""));
	}

	String makeString(int a) {
		int x = a%100;
		return a/100 + (x < 10 ? ".0" : ".") + x;
	}

	public static void main(String[] args) throws Exception {
		banking inst = new banking();
		while (inst.run()) {}
		System.out.println("goodbye");
	}

	StringTokenizer st = new StringTokenizer("");
	BufferedReader input = new BufferedReader(new InputStreamReader(System.in));

	String nextToken() {
		try {
			while (!st.hasMoreTokens()) st = new StringTokenizer(input.readLine());
		} catch (IOException ex) {
			throw new RuntimeException(ex);
		}
		return st.nextToken();
	}

	int nextInt() throws Exception {
		return Integer.parseInt(nextToken());
	}
}
