#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <math.h>

/*
 * CTU Open Contest 2008
 * Solution: FENCE
 * Josef Cibulka, Oct 2008  
 */   

#define MAXPT 20
#define SQR(x) ((x)*(x))


typedef struct Point
{
	int x,y;
}Point;

int n;
Point pt[MAXPT+1];
int v[MAXPT+1], l[MAXPT+1];
int best;

double count_dist(Point *pta, Point *ptb)
{
	return (sqrt(SQR(pta->x - ptb->x) + SQR(pta->y - ptb->y)));
}


int point_cmp(const void *e1, const void *e2)
{
	const Point *p1 = (const Point *)e1;
	const Point *p2 = (const Point *)e2;
	if (p1->x > p2->x) return 1;
	if (p1->x < p2->x) return -1; 
	if (p1->y > p2->y) return 1;
	if (p1->y < p2->y) return -1; 
	else return 0;
}


double count_conv_hull_len(int ptcnt, Point *chpt)
{
	int i,tmp,tmp2;
	int lower[MAXPT+1],upper[MAXPT+1],lcnt,ucnt;
	int tmpd;
	double ret;

	if(ptcnt==0 || ptcnt==1) return 0;
	if(ptcnt==2) return 2*count_dist(&chpt[0], &chpt[1]);

	qsort(chpt,ptcnt,sizeof(chpt[0]),point_cmp);
	upper[0]=0; lower[0]=0;
	upper[1]=1; lower[1]=1; lcnt=2; ucnt=2;
	for(i=2;i<ptcnt;i++)
	{
		while(1)
		{
			tmp=upper[ucnt-1]; tmp2=upper[ucnt-2];
			if(chpt[tmp].x != chpt[tmp2].x)
			{
				tmpd = chpt[tmp].y*(chpt[i].x-chpt[tmp2].x)+chpt[tmp2].y*(chpt[tmp].x-chpt[i].x);
			}
			if(chpt[tmp].x==chpt[tmp2].x || tmpd>=chpt[i].y*(chpt[tmp].x-chpt[tmp2].x))
			{
				upper[ucnt]=i;  ucnt++;
				break;
			} else ucnt--;
			if(ucnt==1)
			{
				ucnt=2; upper[1]=i; break;
			}
		}
		while(1)
		{
			tmp=lower[lcnt-1]; tmp2=lower[lcnt-2];
			if(chpt[tmp].x != chpt[tmp2].x)
			{
				tmpd=chpt[tmp].y*(chpt[i].x-chpt[tmp2].x)+chpt[tmp2].y*(chpt[tmp].x-chpt[i].x);
			}
			if(chpt[tmp].x!=chpt[tmp2].x && tmpd<=chpt[i].y*(chpt[tmp].x-chpt[tmp2].x))
			{
				lower[lcnt]=i; lcnt++;
				break;
			} else lcnt--;
			if(lcnt==1)
			{
				lcnt=2; lower[1]=i; break;
			}
		}
	}
	ret=0;
	for(i=1;i<lcnt;i++)
	{
		ret += count_dist(&chpt[lower[i-1]], &chpt[lower[i]]);
	}
	for(i=1;i<ucnt;i++)
	{
		ret += count_dist(&chpt[upper[i-1]], &chpt[upper[i]]);
	}
	return ret;
}

Point selpt[MAXPT];
Point selptcp[MAXPT];
int selptcnt;
int val_sum, len_sum;
int res;

void select_bt(int hl)
{
	int i;
	double tmp;

	if(hl==n)
	{
		for(i=0;i<selptcnt;i++) selptcp[i] = selpt[i];
		tmp = count_conv_hull_len(selptcnt, selptcp);
		if(tmp <= len_sum)
		{
			if(res<=0 || val_sum<res) res = val_sum;
		}
		return;
	}

	val_sum += v[hl];
	len_sum += l[hl];
	select_bt(hl+1);
	val_sum -= v[hl];
	len_sum -= l[hl];

	selpt[selptcnt++] = pt[hl];
	select_bt(hl+1);
	selptcnt--;	
}

int main(void)
{
	int i;
	
	while(1)
	{
		scanf("%d ", &n);
		if(n==0) break;
		for(i=0;i<n;i++) scanf("%d %d %d %d ", &pt[i].x, &pt[i].y, &v[i], &l[i]);
		selptcnt = 0;
		val_sum = len_sum = 0;
		res = -1;
		select_bt(0);
		printf("The lost value is %d.\n", res);
	}
	return 0;
}
