#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#define MAXN 1000000

/*
 * CTU Open Contest 2008
 * Solution: GAMBLING
 * Josef Cibulka, Oct 2008  
 */   

int n;
int dist;
/* best_dcover[i] ... best covering of i and i+dist by a single arc */
int best_dcover[MAXN+2];
int arc_len;
int price[MAXN+2];
int arc_sum[MAXN+2];
/* arc_sum[i] ... sum of prics on arc starting at i */


typedef struct HeapElement
{
	int val, where, deleted;
} HeapElement;

int heap_size;
struct HeapElement heap[MAXN+2];
int ptr_to_heap[MAXN+2];

int compare_heap_elements(HeapElement el1, HeapElement el2)
{
	if(el1.val<el2.val) return 1;
	return -1;
}

void swap_heap_elements(int h1, int h2)
{
	HeapElement el;
	
	el = heap[h1]; heap[h1] = heap[h2]; heap[h2] = el;
	ptr_to_heap[heap[h1].where] = h1; ptr_to_heap[heap[h2].where] = h2;
}

int heap_delete_min_get()
{
	assert(heap_size > 0);
	if(heap[1].deleted) return -1;
	else return heap[1].val;
}	

int heap_delete_min_delete()
{
	int i, tmpel;
	int ret;
	HeapElement el0;
	
	assert(heap_size > 0);
	ret = heap[1].val	;
	if(heap[1].deleted) ret = -1;
	heap_size--;
	if(heap_size==0) return ret;
	
	heap[1] = heap[heap_size+1];
	ptr_to_heap[heap[1].where] = 1;
	for(i=1;i<=heap_size;)
	{
		el0 = heap[i];
		if(2*i>heap_size) break;
		if(2*i+1>heap_size || compare_heap_elements(heap[2*i], heap[2*i+1])>=0) tmpel = 2*i;
		else tmpel = 2*i+1;

		if(compare_heap_elements(heap[tmpel], el0)>0) 
		{ 
			heap[i] = heap[tmpel]; heap[tmpel] = el0; 
			ptr_to_heap[heap[i].where] = i; ptr_to_heap[heap[tmpel].where] = tmpel;
		}
		else break;
		i = tmpel;
	}
	return ret;
}

int heap_get_min()
{
	int tmp;
	do
	{
		tmp = heap_delete_min_get();
		if(tmp==-1) heap_delete_min_delete();
	}	while(tmp==-1);
	return tmp;
}
void heap_delete(int where)
{
	heap[ptr_to_heap[where]].deleted = 1;
}

void heap_add(int val, int where)
{
	int i;
	heap_size++;
	heap[heap_size].val = val;
	heap[heap_size].where = where;
	heap[heap_size].deleted = 0;
	ptr_to_heap[where] = heap_size;
	for(i=heap_size;i>=1;i>>=1)
	{
		if((i>>1) < 1) break;
		if(compare_heap_elements(heap[i>>1], heap[i])>=0) break;
		swap_heap_elements(i,i>>1);
	}
}

void heap_init()
{
	heap_size = 0;
}



void count_best_dcover(int dist)
{
	int i, tmp;
	
	heap_init();
	for(i=n+dist-arc_len+1;i<n;i++) 
	{
		heap_add(arc_sum[i], i);
	}
	heap_add(arc_sum[0], 0);
	best_dcover[0] = heap_get_min();
	for(i=1;i<n;i++) 
	{
		tmp = (n+dist-arc_len+i)%n;
		heap_delete(tmp);
		heap_add(arc_sum[i], i);
		best_dcover[i] = heap_get_min();
	}
}

int main(void)
{
	int i,tmp;
	int dist;
	int best;
	
	while(1)
	{
		scanf("%d ", &n);
		if(n==0) break;
		for(i=0;i<n;i++) scanf(" %d ", &price[i]);
		arc_len = (n-1)/2;
		dist = arc_len/2;
		
		tmp = 0;
		for(i=0; i<arc_len; i++) tmp+=price[i];
		for(i=0; i<n; i++)
		{
			arc_sum[i] = tmp;
			tmp-=price[i]; tmp+=price[(i+arc_len)%n];
		}

		count_best_dcover(dist);


		best = -1;
		for(i=0;i<n;i++) 
		{
			tmp = arc_sum[i] + best_dcover[(i+arc_len)%n] + best_dcover[(n+i-1-dist)%n];
			if(best==-1 || tmp<best) best = tmp;
		}
		printf("%d\n", best);
	}
		
	return 0;
}

