/*
 * CTU Open 2009
 * Sample Solution: Robotic Rails
 * Martin Kacer
 */

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.SortedSet;
import java.util.StringTokenizer;
import java.util.TreeSet;

public class rr {

/**
 * Simple rational number.
 */
static class Rational implements Comparable<Rational> {
	public static final Rational ZERO = new Rational(0,1);
	public static final Rational ONE = new Rational(1,1);
	int num, den;
	public Rational(int num, int den) {
		int gcd = gcd(num, den);
		this.num = num / gcd;
		this.den = den / gcd;
	}
	public int compareTo(Rational o) {
		long me = ((long)num) * o.den;
		long other = ((long)o.num) * den;
		return (me < other) ? -1 : (me > other) ? +1 : 0;
	}
	@Override
	public boolean equals(Object obj) {
		if (obj == this) return true;
		if (!(obj instanceof Rational)) return false;
		Rational robj = (Rational) obj;
		return (num == robj.num) && (den == robj.den);
	}
	@Override
	public int hashCode() {
		return num * 313 + den;
	}
	@Override
	public String toString() {
		return num + "/" + den;
	}
	static final int gcd(int a, int b) {
		return (a > b) ? gcd2(a, b) : gcd2(b, a);
	}
	private static final int gcd2(int a, int b) {
		return (b == 0) ? a : gcd2(b, a % b);
	}
}

/**
 * One line segment in one direction.
 * Each input segment is created twice (for both directions).
 */
static class Line {
	int x1, y1,  // first point
		x2, y2,  // second point
		xd, yd;  // direction
	double len;
	
	/**
	 * Intersections of this segment with others.
	 * Only those intersections are contained, which can be used
	 * to go onto the other line in the respective direction.
	 */
	SortedSet<Intersection> intersections = new TreeSet<Intersection>();

	public Line(int x1, int y1, int x2, int y2) {
		this.x1 = x1;
		this.y1 = y1;
		this.x2 = x2;
		this.y2 = y2;
		this.xd = x2 - x1;
		this.yd = y2 - y1;
		this.len = Math.sqrt(xd*xd + yd*yd);
	}
	public String toString() {
		return "["+x1+","+y1+"]--["+x2+","+y2+"]";
	}
}

static class Intersection implements Comparable<Intersection> {
	Line otherLine;
	Rational myPos, otherPos; // position on the lines, 0--1
	public Intersection(Rational pos, Line line, Rational pos2) {
		myPos = pos;
		otherLine = line;
		otherPos = pos2;
	}
	public int compareTo(Intersection o) {
		int res = myPos.compareTo(o.myPos);
		// tie-breaker
		return (res != 0) ? res : otherLine.hashCode() - o.otherLine.hashCode();
	}
}

/**
 * One "node" in the graph of possible states.
 * NOTE: compareTo and equals methods are not consistent!
 * equals - considers position only
 * compareTo - according to the distance
 */
static class Position implements Comparable<Position> {
	double dist; // total distance from the starting point
	Line line; // on which line we are just now
	Rational pos; // position on the line 0--1
	boolean closed;

	public Position(double dist, Line line, Rational pos) {
		this.dist = dist;
		this.line = line;
		this.pos = pos;
	}

	public int hashCode() {
		return line.hashCode() * 31 + pos.hashCode();
	}

	public boolean equals(Object obj) {
		if (obj == this) return true;
		if (!(obj instanceof Position)) return false;
		Position pobj = (Position) obj;
		return line.equals(pobj.line) && pos.equals(pobj.pos);
	}

	public String toString() {
		return "pos["+(line.x1+(double)line.xd*pos.num/pos.den)
			+","+(line.y1+(double)line.yd*pos.num/pos.den)
			+"]->" + line.xd + "," + line.yd
			+ "/d=" + dist;
	}

	public int compareTo(Position o) {
		int res = Double.compare(dist, o.dist);
		if (res != 0) return res;
		if (equals(o)) return 0;
		if (line == o.line) return pos.compareTo(o.pos);
		return line.hashCode() - o.line.hashCode();
	}
}

	static final NumberFormat NUM_FORM = new DecimalFormat("0.000");
	static final double EPS = 1e-7;
	StringTokenizer st = new StringTokenizer("");
	BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
	public static void main(String[] args) throws Exception {
		rr instance = new rr();
		while (instance.run()) {/*nothing*/}
	}
	String nextToken() throws Exception {
		while (!st.hasMoreTokens()) st = new StringTokenizer(input.readLine());
		return st.nextToken();
	}
	int nextInt() throws Exception {
		return Integer.parseInt(nextToken());
	}

	Line[] lines;
	Position goal;
	SortedSet<Position> queue = new TreeSet<Position>();
	Map<Position,Position> visited = new HashMap<Position,Position>();
	int probCounter = 0;

	/**
	 * Solve on situation.
	 */
	boolean run() throws Exception {
		int n = nextInt();
		if (n == 0) return false;
		queue.clear();
		visited.clear();
		lines = new Line[n*2];
		for (int i = 0; i < n; ++i) {
			// remember the line in both directions
			int x1 = nextInt(), y1 = nextInt(), x2 = nextInt(), y2 = nextInt();
			lines[i*2] = new Line(x1, y1, x2, y2);
			lines[i*2+1] = new Line(x2, y2, x1, y1);
		}
		goal = new Position(0.0, lines[lines.length-2], Rational.ONE);
		for (int i = 0; i < lines.length; ++i)
			for (int j = 0; j < lines.length; ++j) {
				// create the list of intersections
				intersect(lines[i], lines[j]);
			}
		// add a virtual "intersection" at the end of the final segment
		goal.line.intersections.add(new Intersection(Rational.ONE, goal.line, Rational.ONE));
		
		dijkstra();
		return true;
	}

	/**
	 * Find the shortest path.
	 */
	void dijkstra() {
		Position pos = new Position(0.0, lines[0], Rational.ZERO);
		queue.add(pos);
		visited.put(pos,pos);
		while (!queue.isEmpty()) {
			pos = queue.first();
			queue.remove(pos);
			if (pos.equals(goal)) {
				break;
			}
			for (Intersection x : pos.line.intersections.tailSet(new Intersection(pos.pos, pos.line, Rational.ZERO))) {
				// ignore everything that is on the same intersection - important!
				if (pos.pos.compareTo(x.myPos) == 0) continue;
				enqueue(pos, x);
			}
		}
		//System.out.print("Robot #" + (++probCounter) + ": ");
		if (pos.equals(goal)) {
			System.out.println(NUM_FORM.format(pos.dist));
		} else {
			System.out.println("unreachable");
		}
	}

	void enqueue(Position pos, Intersection x) {
		double d1 = pos.line.len * pos.pos.num / pos.pos.den;
		double d2 = pos.line.len * x.myPos.num / x.myPos.den;
		Position next = new Position(pos.dist + d2-d1, x.otherLine, x.otherPos);
		Position exist = visited.get(next);
		if (exist == null) {
			visited.put(next,next);
			queue.add(next);
		} else if (next.dist < exist.dist) {
			// only change distance if it is better than before
			queue.remove(exist);
			exist.dist = next.dist;
			queue.add(exist);
		}
		//System.out.println("  debug: " + pos + " -> " + next);
	}

	/**
	 * Find an intersection of the two lines.
	 * Add it to the list.
	 */
	void intersect(Line a, Line b) {
		int d = a.yd * b.xd - a.xd * b.yd;
		int da = (a.x1 - b.x1) * b.yd - (a.y1 - b.y1) * b.xd;
		int db = (a.x1 - b.x1) * a.yd - (a.y1 - b.y1) * a.xd;
		if (d == 0) {
			if (da != 0 || db != 0) return; // parallel
			if (a.xd == 0)
				intersectColinear(a, b, a.y1, a.y2, b.y1, b.y2);
			else
				intersectColinear(a, b, a.x1, a.x2, b.x1, b.x2);
		} else {
			Rational ta = new Rational(da, d), tb = new Rational(db, d);
			if (ta.compareTo(Rational.ZERO) >= 0 && ta.compareTo(Rational.ONE) <= 0
					&& tb.compareTo(Rational.ZERO) >= 0 && tb.compareTo(Rational.ONE) <= 0
					&& checkAngle(a.xd,a.yd, b.xd,b.yd)) {
				a.intersections.add(new Intersection(ta, b, tb));
			}
		}
	}

	/**
	 * Handle colinear segments.
	 * Consider two "intersections" at the endpoints.
	 */
	void intersectColinear(Line a, Line b, int a1, int a2, int b1, int b2) {
		if (a1 > a2) {
			intersectColinear(a, b, a2, a1, b2, b1);
			return;
		}
		if (b1 > b2) return; // oposite directions, no u-turn possible
		if (a1 <= b1 && b1 <= a2) {
			a.intersections.add(new Intersection(new Rational(b1-a1,a2-a1), b, Rational.ZERO));
		}
		if (b1 <= a2 && a2 <= b2) {
			a.intersections.add(new Intersection(Rational.ONE, b, new Rational(a2-b1,b2-b1)));
		}
	}

	/**
	 * Check whether the angle is at most 90 degrees.
	 */
	boolean checkAngle(int x1, int y1, int x2, int y2) {
		int xp = -y1, yp = x1;
		return xp*y2 <= yp*x2;
	}

	// not used!
	void debugDumpIntersections() {
		for (int i = 0; i < lines.length; ++i) {
			for (Intersection x : lines[i].intersections) {
				System.out.println(lines[i] + " x " + x.otherLine
						+ " " + x.myPos + "/" + x.otherPos);
			}
		}
	}
}
