#define NDEBUG

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

const int MAX_PROG_C = 20;
const int MAX_PROG_L = 50010;
const int MOD        = 1000000;

int computeStrHash(char *c,int iL);
int advanceStrHash(int iHash,char *code,int iL);

class CommonCode {
	char *code;
	int iL;
public:
	CommonCode(char *code,int iL) {
		this->code = code;
		this->iL   = iL;
	}
	void print() {
		for (int i=0;i<iL;i++)
			printf("%c",code[i]);
		printf("\n");
	}
	int computeHash() {
		int iR = computeStrHash(getCode(),getCodeL());
        	return iR;
	}
	int getCodeL() {
		return iL;
	}
	char* getCode() {
		return code;
	}
};

class CommonCodeList {
	class Node {
		Node *next;
		CommonCode c;
	public:
        	Node(CommonCode c1): c(c1) {
			next = NULL;
		}
		Node *getNext() {
			return next;
		}
		void setNext(Node *n) {
			next = n;
		}
		CommonCode getCode() {
			return c;
		}
		~Node() {
			if (next != NULL)
				delete next;
		}
	};

	Node *root;
	Node *last;
	int iL;
public:

	class Iterator {
		Node *cur;
	public:
        	Iterator(Node *cur) {
			this->cur = cur;
		}
        	bool hasNext() {
			return cur->getNext() != NULL;
		}
		CommonCode next() {
			cur = cur->getNext();
			return cur->getCode();
		}
	};

	Iterator getIterator() {
        	return Iterator(root);
	}
	CommonCodeList() {
		//Initialize the sentinel
		root = new Node(CommonCode(NULL,0));
		last = root;
		iL   = 0;
	}
	void add(CommonCode c) {
		last->setNext(new Node(c));
		last = last->getNext();
		iL++;
	}
	void clear() {
		iL = 0;
		last = root;
                if (root->getNext())
			delete root->getNext();
	}
        int getL() {
		return iL;
	}
	~CommonCodeList() {
		delete root;
	}
};

class Program {
	char code[MAX_PROG_L];
	int iProgL;

	bool validShift(CommonCode c,int iS) {
        	int iL = c.getCodeL();
		for (int i=0;i<iL;i++)
			if  (c.getCode()[i] != code[iS+i])
				return false;

		return true;
	}
public:
	Program(FILE *fin) {
		fscanf(fin,"%s",code);
		iProgL = strlen(code);
	}
	int getCodeL() {
		return iProgL;
	}
	char *getCode() {
		return code;
	}
	bool contains(CommonCode c) {
		int iCandidateCodeL = c.getCodeL();
                int iCandidateHash  = c.computeHash();
		int iCurHash        = computeStrHash(code,iCandidateCodeL);

		for (int iCurShift=0;iCurShift+iCandidateCodeL<iProgL;iCurShift++) {
			if (iCurHash == iCandidateHash) {
				if (validShift(c,iCurShift))
					return true;
			}
			if (iCurShift+iCandidateCodeL <= iProgL - 1)
				iCurHash = advanceStrHash(iCurHash,code+iCurShift,iCandidateCodeL);
		}

		return false;
        }
};

Program* prog[MAX_PROG_C];
CommonCodeList list;
int iProgC;
int iPow16[10];

int hex(char c) {
	if (c>='0' && c<='9')
		return c - '0';
	else
		return 10 + c - 'A';
}

int pow16(int iPow) {
	return iPow16[iPow % 10];
}

int abs(int i) {
	return i > 0 ? i : -i;
}

int advanceStrHash(int iHash,char *cur,int iL) {
	int iR = iHash;

	iR = abs( iR - hex(cur[0])*pow16(iL-1) ) % MOD;
        iR = ( iR*pow16(1) ) % MOD;
        iR = ( iR + hex(cur[iL]) ) % MOD;

	return iR;
}

int computeStrHash(char *c,int iL) {
	int iR = 0;
        int iPow = 0;

	for (int iPos = iL-1;iPos>=0;iPos--,iPow++) {
        	iR = (iR + hex(c[iPos])*pow16(iPow)) % MOD;
	}

	return iR;
}

FILE* openInput() {
        #ifndef NDEBUG
		return fopen("virus.in","r");
	#else
		return stdin;
	#endif
}

void load() {
	FILE *fin = openInput();

	fscanf(fin,"%d",&iProgC);
        for (int i=0;i<iProgC;i++)
                prog[i] = new Program(fin);

	fclose(fin);
}

int cmp(const void *e1,const void *e2) {
	Program *p1 = *((Program**)e1);
	Program *p2 = *((Program**)e2);

	return p1->getCodeL() - p2->getCodeL();
}

void sort() {
	qsort(prog,iProgC,sizeof(Program*),cmp);
}

int min(int i1,int i2) {
	return i1 < i2 ? i1 : i2;
}

int max(int i1,int i2) {
	return i1 > i2 ? i1 : i2;
}

CommonCode findCommonCode(char *code1,char *code2) {
	int iL;
	for (iL=0;true;iL++) {
		if (code1[iL] != code2[iL])
			break;
		if (min(code1[iL],code2[iL]) == 0)
			break;
	}
	return CommonCode(code1,iL);
}

void findCandidateViruses() {
	int iL1     = prog[0]->getCodeL();
        char *code1 = prog[0]->getCode();
	int iL2     = prog[1]->getCodeL();
	char *code2 = prog[1]->getCode();
	int iMinL   = iL1/10 - 1;
//	int iMaxL   = 0;

        for (int iStart1=0;iStart1 + iMinL < iL1;iStart1++)
		for (int iStart2=0;iStart2 + iMinL < iL2;iStart2++) {
                	CommonCode c = findCommonCode(code1 + iStart1,
                        			      code2 + iStart2);
			//if (iMaxL < c.getCodeL()) {
			//	iMaxL = c.getCodeL();
			//	list.clear();
			//	list.add(c);
			//} else
			//if (iMaxL == c.getCodeL()) {
			if (c.getCodeL() > 0)
				list.add(c);
                        //}
		}

	//printf("%d\n",iMaxL);
}

void printCandidateViruses() {
	printf("%d candidate viruses found:\n",list.getL());
	CommonCodeList::Iterator iter = list.getIterator();

	while (iter.hasNext()) {
		iter.next().print();
	}
}

void findRealVirus() {
	CommonCodeList::Iterator iter = list.getIterator();
	int iMaxL = 0;
	CommonCode cVirus(NULL,0);

	while (iter.hasNext()) {
		CommonCode c = iter.next();

		int i=0;
                for (i=0;i<iProgC;i++)
			if (!prog[i]->contains(c))
				break;

        	if (i == iProgC) {
			if (c.getCodeL() > iMaxL) {
				iMaxL = c.getCodeL();
				cVirus = c;
			}
		}
	}
	cVirus.print();
}

void solve() {
	sort();
	findCandidateViruses();
	//printCandidateViruses();
	findRealVirus();
}

void init() {
	int iPow = 1;

	for (int i=0;i<10;i++) {
		iPow16[i] = iPow;
		iPow = (iPow*16) % MOD;
	}
}

void freeDS() {
	for (int i=0;i<iProgC;i++)
		delete prog[i];
}

int main() {
	init();
	load();
	solve();
	freeDS();

	return 0;
}
